#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"

#include <stdio.h>
#include <stdlib.h>

#define SYSFS_DIR "/sys/devices/platform/s3c2410-i2c/i2c-adapter/i2c-0/0-0008"
#define N_SAMPLES 48
#define N_AVG 3
typedef struct battgrapht_ {
  GtkWidget * widget;
  GdkPixmap * pixmap;
  gfloat data[N_SAMPLES];
  int n, w;
  gfloat min, max;
} battgraph_t;

typedef struct mokostatst_ {
  GtkWidget * temp;
  GtkWidget * voltage;
  GtkWidget * current;
  GtkWidget * mode;
  GtkWidget * state;

  char temp_val[64];
  char voltage_val[64];
  char current_val[64];
  char mode_val[64];
  char state_val[64];

  battgraph_t * temp_graph;
  battgraph_t * voltage_graph;
  battgraph_t * current_graph;

} mokostats_t;

int read_file(char * filename, char * buf)
{
  FILE * f;
  char fname[256];

  sprintf(fname, "%s/%s", SYSFS_DIR, filename);

  f = fopen(fname, "r");
  if(!f) return -1;
  if(strcmp(filename, "chgstate"))
    fscanf(f, "%s\n", buf);
  else 
    fscanf(f, "<NULL> %s", buf);
  fclose(f);
}

int add_data(battgraph_t * graph, gfloat data) 
{
  graph->data[graph->w++] = data;

  if (graph->n < (N_SAMPLES)) graph->n++ ;

  if (graph->w > (N_SAMPLES-1)) graph->w = 0;

  //printf("Added data... %f\n", data);
}

gfloat get_value(battgraph_t * g, int time)
{
  if(time > 0) {
    printf("Error cannot see forward in time !!!\n");
    return g->min;
  }
  
  if((g->n + time) < 0) {
    //printf("Not enough data...\n");
    return g->min;
  }

  int index = (g->w + time) >= 0 ? (g->w + time) : (N_SAMPLES-1) + (g->w+time);
 
  //printf("n=%d w=%d data[%d] asked %i  ", g->n, g->w, index, time);

  return g->data[index];

}

int y_val(battgraph_t * graph, int index)
{
  int val, i;
  int hist;
  //return graph->widget->allocation.height * (1.0 - ((get_value(graph, index) - graph->min) / (graph->max - graph->min)));
  


  val = 0;
  hist = (index + N_SAMPLES > N_AVG) ? N_AVG : index + N_SAMPLES ; 

  for (i=0; i<hist; i++) {
    //printf("%f ", get_value(graph, index-i));
    //printf("adding: %f\n", (1.0 - ((get_value(graph, index - i) - graph->min) / (graph->max - graph->min))));
    val += graph->widget->allocation.height * (1.0 - ((get_value(graph, index - i) - graph->min) / (graph->max - graph->min)));
  }
  val /= hist;

  //printf("y_val(%d) = %i\n", index, val);

  return val;
}

int update_graph(battgraph_t * graph)
{
  int i;
  int x, y, x1, y1;


  if(graph->n < N_AVG) return;

  /* Clear graph */
  gdk_draw_rectangle( (GdkDrawable*)graph->pixmap, graph->widget->style->black_gc, TRUE, 0, 0,
		      graph->widget->allocation.width, graph->widget->allocation.height );  
  
  x = graph->widget->allocation.width;
  y = y_val(graph, 0);
  
  for (i=-1; i>=(-N_SAMPLES+N_AVG);i--) {
    gdk_draw_line((GdkDrawable*)graph->pixmap, graph->widget->style->white_gc,
		  x, y,
		  x1 = ((N_SAMPLES-N_AVG+i) * graph->widget->allocation.width) / (N_SAMPLES-N_AVG-1), y1 = y_val(graph, i)); 
    //printf("(%i,%i)(%i,%i)\n", x, y, x1, y1);
    x = x1; 
    y = y1;
  }
  //printf("Next...\n");


#if 0

  if(graph->n < (N_SAMPLES-1)) {
    for (i=1; i < graph->n; i++) {
      gdk_draw_line((GdkDrawable*)graph->pixmap, graph->widget->style->white_gc,
		    x = ((i-1) * graph->widget->allocation.width) / (N_SAMPLES-1), y = y_val(graph, i-1),
		    x1 = ((i) * graph->widget->allocation.width) / (N_SAMPLES-1), y1 = y_val(graph, i));
      //printf("(%i,%i),(%i,%i) - (%i,%f)(%i,%f)\n", x, y, x1, y1, i-1, graph->data[i-1], i, graph->data[i]);
    }
  } else {
    
    for(i=1; i < N_SAMPLES + 1; i++) {
      int j = (i+graph->w-1)%N_SAMPLES;

      gdk_draw_line((GdkDrawable*)graph->pixmap, graph->widget->style->white_gc,
		    ((i-1) * graph->widget->allocation.width) / (N_SAMPLES-1), y_val(graph, j-1),
		    ((i) * graph->widget->allocation.width) / (N_SAMPLES-1), y_val(graph, j));      
    }
  }
#endif
  //printf("Will draw the window....\n");

  gdk_draw_pixmap (graph->widget->window,
		   graph->widget->style->fg_gc[GTK_WIDGET_STATE (graph->widget)],
		   (GdkDrawable *)graph->pixmap,
		   0, 0,
		   0, 0,
		   -1, -1);

  // printf("Drawed.....\n");
}

gboolean batt_update_cb(mokostats_t * st)
{

  read_file("battemp", st->temp_val);
  read_file("battvolt", st->voltage_val);
  read_file("chgcur", st->current_val);
  read_file("chgmode", st->mode_val);
  read_file("chgstate", st->state_val);

  //printf("Read values. %s %s %s %s %s \n", st->temp_val, st->voltage_val, st->current_val, st->mode_val, st->state_val);

  int milivolts  = atoi(st->voltage_val);
  
  gfloat val = (gfloat)milivolts / 1000.0;
  
  add_data(st->voltage_graph, val);
  
  gfloat temp = atof(st->temp_val);
  add_data(st->temp_graph, temp);

  if (!strcmp(st->mode_val, "idle") || !strcmp(st->mode_val, "pre")) {
    sprintf(st->current_val, "0.0");
  }

  gfloat curr = atof(st->current_val);
  add_data(st->current_graph, curr);


  gtk_progress_bar_set_fraction(st->voltage, val/4.3);
  sprintf(st->voltage_val, "%.3fv", val);
  gtk_progress_bar_set_text(st->voltage, st->voltage_val);

  gtk_entry_set_text(st->temp, st->temp_val); 
  gtk_entry_set_editable(st->temp, FALSE);

  gtk_entry_set_text(GTK_ENTRY(st->current), st->current_val);
  gtk_entry_set_editable(GTK_ENTRY(st->current), FALSE);

  static char pram_text[128];
  sprintf(pram_text, "%s / %s", st->mode_val, st->state_val);

  gtk_entry_set_text(GTK_ENTRY(st->mode), pram_text);
  gtk_entry_set_editable(GTK_ENTRY(st->mode), FALSE);


  update_graph(st->voltage_graph);
  update_graph(st->current_graph);
  update_graph(st->temp_graph);

  return TRUE;
}

void
on_graph_realize                  (GtkWidget       *widget,
                                        gpointer         user_data)
{
  
  
}


gboolean
on_graph_expose_event             (GtkWidget       *widget,
				   GdkEventExpose  *event,
				   gpointer         user_data)
{
  battgraph_t * graph = (battgraph_t *) user_data;

  //printf("in %s w=%d h=%d\n", __func__, widget->allocation.width, widget->allocation.height );
  
  if ( graph->pixmap == NULL ) {
    free(graph->pixmap);
  }
  graph->pixmap = gdk_pixmap_new( widget->window, widget->allocation.width, widget->allocation.height, -1 );
  
  gdk_draw_rectangle( (GdkDrawable*)graph->pixmap, widget->style->black_gc, TRUE, 0, 0,
		      widget->allocation.width, widget->allocation.height );  

  update_graph(graph);
  //gtk_widget_queue_draw(widget);
  
  
  gdk_draw_pixmap (widget->window,
		   widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		   (GdkDrawable *)graph->pixmap,
		   0, 0,
		   0, 0,
		   -1, -1);
  
  return FALSE;
}

int batt_graph_init(GtkWidget * mwin, mokostats_t * st) {
  st->temp_graph = malloc(sizeof(*st->temp_graph));
  if (st->temp_graph == NULL) {
    return -1;
  }
  st->voltage_graph = malloc(sizeof(*st->voltage_graph));
  if(st->voltage_graph == NULL) {
    free(st->temp_graph);
    return -1;
  }
  st->current_graph = malloc(sizeof(*st->current_graph));
  if (st->current_graph == NULL){
    free(st->voltage_graph);
    free(st->temp_graph);
    return -1;
  }
 
  st->temp_graph->widget = lookup_widget(mwin, "graph_temp");
  st->voltage_graph->widget = lookup_widget(mwin, "graph_volt");
  st->current_graph->widget = lookup_widget(mwin, "graph_curr");

  g_signal_connect ((gpointer) st->temp_graph->widget, "realize",
                    G_CALLBACK (on_graph_realize),
                    st->temp_graph);
  g_signal_connect ((gpointer) st->temp_graph->widget, "expose_event",
                    G_CALLBACK (on_graph_expose_event),
                    st->temp_graph);

  g_signal_connect ((gpointer) st->voltage_graph->widget, "realize",
                    G_CALLBACK (on_graph_realize),
                    st->voltage_graph);
  g_signal_connect ((gpointer) st->voltage_graph->widget, "expose_event",
                    G_CALLBACK (on_graph_expose_event),
                    st->voltage_graph);

  g_signal_connect ((gpointer) st->current_graph->widget, "realize",
                    G_CALLBACK (on_graph_realize),
                    st->current_graph);
  g_signal_connect ((gpointer) st->current_graph->widget, "expose_event",
                    G_CALLBACK (on_graph_expose_event),
                    st->current_graph);

  st->temp_graph->n = st->temp_graph->w = st->temp_graph->min = 0;
  st->temp_graph->max = 20.0;

  st->voltage_graph->n = st->voltage_graph->w = 0;
  st->voltage_graph->min = 3.7;
  st->voltage_graph->max = 4.3;

  st->current_graph->n = st->current_graph->w = st->current_graph->min = 0;
  st->current_graph->max = 1000.0;

  return 0;
}

int init_battery(GtkWidget * mwin) {
  mokostats_t * st = malloc(sizeof(*st));

  if(st == NULL) return -1;


  st->temp = lookup_widget(mwin, "temp_entry");
  st->voltage = lookup_widget(mwin, "battgauge");
  st->current = lookup_widget(mwin, "current_entry");
  st->mode = lookup_widget(mwin, "mode_entry");
  
  batt_graph_init(mwin, st);

  gtk_timeout_add(1000, batt_update_cb, st);
  return 0;
}
